@@ -15,8 +15,6 @@
 package casbin
 
 import (
-	"encoding/json"
-	"fmt"
 	"testing"
 
 	"github.com/casbin/casbin/v2/log"
@@ -413,6 +411,18 @@ func (rm *testCustomRoleManager) AddDomainLinkConditionFunc(user string, role st
 func (rm *testCustomRoleManager) SetDomainLinkConditionFuncParams(user string, role string, domain string, params ...string) {
 }
 
+func (rm *testCustomRoleManager) DeleteDomain(domain string) error {
+	return nil
+}
+
+func (rm *testCustomRoleManager) GetImplicitRoles(name string, domain ...string) ([]string, error) {
+	return []string{}, nil
+}
+
+func (rm *testCustomRoleManager) GetImplicitUsers(name string, domain ...string) ([]string, error) {
+	return []string{}, nil
+}
+
 func TestRBACModelWithCustomRoleManager(t *testing.T) {
 	e, _ := NewEnforcer("examples/rbac_model.conf", "examples/rbac_policy.csv")
 	e.SetRoleManager(NewRoleManager())
@@ -429,131 +439,6 @@ func TestRBACModelWithCustomRoleManager(t *testing.T) {
 	testEnforce(t, e, "bob", "data2", "write", true)
 }
 
-type testResource struct {
-	Name  string
-	Owner string
-}
-
-func newTestResource(name string, owner string) testResource {
-	r := testResource{}
-	r.Name = name
-	r.Owner = owner
-	return r
-}
-
-func TestABACModel(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_model.conf")
-
-	data1 := newTestResource("data1", "alice")
-	data2 := newTestResource("data2", "bob")
-
-	testEnforce(t, e, "alice", data1, "read", true)
-	testEnforce(t, e, "alice", data1, "write", true)
-	testEnforce(t, e, "alice", data2, "read", false)
-	testEnforce(t, e, "alice", data2, "write", false)
-	testEnforce(t, e, "bob", data1, "read", false)
-	testEnforce(t, e, "bob", data1, "write", false)
-	testEnforce(t, e, "bob", data2, "read", true)
-	testEnforce(t, e, "bob", data2, "write", true)
-}
-
-func TestABACMapRequest(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_model.conf")
-
-	data1 := map[string]interface{}{
-		"Name":  "data1",
-		"Owner": "alice",
-	}
-	data2 := map[string]interface{}{
-		"Name":  "data2",
-		"Owner": "bob",
-	}
-
-	testEnforce(t, e, "alice", data1, "read", true)
-	testEnforce(t, e, "alice", data1, "write", true)
-	testEnforce(t, e, "alice", data2, "read", false)
-	testEnforce(t, e, "alice", data2, "write", false)
-	testEnforce(t, e, "bob", data1, "read", false)
-	testEnforce(t, e, "bob", data1, "write", false)
-	testEnforce(t, e, "bob", data2, "read", true)
-	testEnforce(t, e, "bob", data2, "write", true)
-}
-
-func TestABACTypes(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_model.conf")
-	matcher := `"moderator" IN r.sub.Roles && r.sub.Enabled == true && r.sub.Age >= 21 && r.sub.Name != "foo"`
-	e.GetModel()["m"]["m"].Value = util.RemoveComments(util.EscapeAssertion(matcher))
-
-	structRequest := struct {
-		Roles   []interface{}
-		Enabled bool
-		Age     int
-		Name    string
-	}{
-		Roles:   []interface{}{"user", "moderator"},
-		Enabled: true,
-		Age:     30,
-		Name:    "alice",
-	}
-	testEnforce(t, e, structRequest, "", "", true)
-
-	mapRequest := map[string]interface{}{
-		"Roles":   []interface{}{"user", "moderator"},
-		"Enabled": true,
-		"Age":     30,
-		"Name":    "alice",
-	}
-	testEnforce(t, e, mapRequest, nil, "", true)
-
-	e.EnableAcceptJsonRequest(true)
-	jsonRequest, _ := json.Marshal(mapRequest)
-	testEnforce(t, e, string(jsonRequest), "", "", true)
-}
-
-func TestABACJsonRequest(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_model.conf")
-	e.EnableAcceptJsonRequest(true)
-
-	data1Json := `{ "Name": "data1", "Owner": "alice"}`
-	data2Json := `{ "Name": "data2", "Owner": "bob"}`
-
-	testEnforce(t, e, "alice", data1Json, "read", true)
-	testEnforce(t, e, "alice", data1Json, "write", true)
-	testEnforce(t, e, "alice", data2Json, "read", false)
-	testEnforce(t, e, "alice", data2Json, "write", false)
-	testEnforce(t, e, "bob", data1Json, "read", false)
-	testEnforce(t, e, "bob", data1Json, "write", false)
-	testEnforce(t, e, "bob", data2Json, "read", true)
-	testEnforce(t, e, "bob", data2Json, "write", true)
-
-	e, _ = NewEnforcer("examples/abac_not_using_policy_model.conf", "examples/abac_rule_effect_policy.csv")
-	e.EnableAcceptJsonRequest(true)
-
-	testEnforce(t, e, "alice", data1Json, "read", true)
-	testEnforce(t, e, "alice", data1Json, "write", true)
-	testEnforce(t, e, "alice", data2Json, "read", false)
-	testEnforce(t, e, "alice", data2Json, "write", false)
-
-	e, _ = NewEnforcer("examples/abac_rule_model.conf", "examples/abac_rule_policy.csv")
-	e.EnableAcceptJsonRequest(true)
-	sub1Json := `{"Name": "alice", "Age": 16}`
-	sub2Json := `{"Name": "alice", "Age": 20}`
-	sub3Json := `{"Name": "alice", "Age": 65}`
-
-	testEnforce(t, e, sub1Json, "/data1", "read", false)
-	testEnforce(t, e, sub1Json, "/data2", "read", false)
-	testEnforce(t, e, sub1Json, "/data1", "write", false)
-	testEnforce(t, e, sub1Json, "/data2", "write", true)
-	testEnforce(t, e, sub2Json, "/data1", "read", true)
-	testEnforce(t, e, sub2Json, "/data2", "read", false)
-	testEnforce(t, e, sub2Json, "/data1", "write", false)
-	testEnforce(t, e, sub2Json, "/data2", "write", true)
-	testEnforce(t, e, sub3Json, "/data1", "read", true)
-	testEnforce(t, e, sub3Json, "/data2", "read", false)
-	testEnforce(t, e, sub3Json, "/data1", "write", false)
-	testEnforce(t, e, sub3Json, "/data2", "write", false)
-}
-
 func TestKeyMatchModel(t *testing.T) {
 	e, _ := NewEnforcer("examples/keymatch_model.conf", "examples/keymatch_policy.csv")
 
@@ -690,56 +575,6 @@ func TestRBACModelInMultiLines(t *testing.T) {
 	testEnforce(t, e, "bob", "data2", "write", true)
 }
 
-type testSub struct {
-	Name string
-	Age  int
-}
-
-func newTestSubject(name string, age int) testSub {
-	s := testSub{}
-	s.Name = name
-	s.Age = age
-	return s
-}
-
-func TestABACNotUsingPolicy(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_not_using_policy_model.conf", "examples/abac_rule_effect_policy.csv")
-	data1 := newTestResource("data1", "alice")
-	data2 := newTestResource("data2", "bob")
-
-	testEnforce(t, e, "alice", data1, "read", true)
-	testEnforce(t, e, "alice", data1, "write", true)
-	testEnforce(t, e, "alice", data2, "read", false)
-	testEnforce(t, e, "alice", data2, "write", false)
-}
-
-func TestABACPolicy(t *testing.T) {
-	e, _ := NewEnforcer("examples/abac_rule_model.conf", "examples/abac_rule_policy.csv")
-	m := e.GetModel()
-	for sec, ast := range m {
-		fmt.Println(sec)
-		for ptype, p := range ast {
-			fmt.Println(ptype, p)
-		}
-	}
-	sub1 := newTestSubject("alice", 16)
-	sub2 := newTestSubject("alice", 20)
-	sub3 := newTestSubject("alice", 65)
-
-	testEnforce(t, e, sub1, "/data1", "read", false)
-	testEnforce(t, e, sub1, "/data2", "read", false)
-	testEnforce(t, e, sub1, "/data1", "write", false)
-	testEnforce(t, e, sub1, "/data2", "write", true)
-	testEnforce(t, e, sub2, "/data1", "read", true)
-	testEnforce(t, e, sub2, "/data2", "read", false)
-	testEnforce(t, e, sub2, "/data1", "write", false)
-	testEnforce(t, e, sub2, "/data2", "write", true)
-	testEnforce(t, e, sub3, "/data1", "read", true)
-	testEnforce(t, e, sub3, "/data2", "read", false)
-	testEnforce(t, e, sub3, "/data1", "write", false)
-	testEnforce(t, e, sub3, "/data2", "write", false)
-}
-
 func TestCommentModel(t *testing.T) {
 	e, _ := NewEnforcer("examples/comment_model.conf", "examples/basic_policy.csv")
 	testEnforce(t, e, "alice", "data1", "read", true)
@@ -853,3 +688,21 @@ func TestTemporalRolesModelWithDomain(t *testing.T) {
 	testDomainEnforce(t, e, "alice", "domain_not_exist", "data8", "read", false)
 	testDomainEnforce(t, e, "alice", "domain_not_exist", "data8", "write", false)
 }
+
+func TestReBACModel(t *testing.T) {
+	e, _ := NewEnforcer("examples/rebac_model.conf", "examples/rebac_policy.csv")
+
+	testEnforce(t, e, "alice", "doc1", "read", true)
+	testEnforce(t, e, "alice", "doc1", "write", false)
+	testEnforce(t, e, "alice", "doc2", "read", false)
+	testEnforce(t, e, "alice", "doc2", "write", false)
+	testEnforce(t, e, "alice", "doc3", "read", false)
+	testEnforce(t, e, "alice", "doc3", "write", false)
+
+	testEnforce(t, e, "bob", "doc1", "read", false)
+	testEnforce(t, e, "bob", "doc1", "write", false)
+	testEnforce(t, e, "bob", "doc2", "read", true)
+	testEnforce(t, e, "bob", "doc2", "write", false)
+	testEnforce(t, e, "bob", "doc3", "read", false)
+	testEnforce(t, e, "bob", "doc3", "write", false)
+}
