@@ -15,6 +15,7 @@
 package defaultrolemanager
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 	"sync"
@@ -203,6 +204,7 @@ type RoleManagerImpl struct {
 	domainMatchingFunc rbac.MatchingFunc
 	logger             log.Logger
 	matchingFuncCache  *util.SyncLRUCache
+	mutex              sync.Mutex
 }
 
 // NewRoleManagerImpl is the constructor for creating an instance of the
@@ -349,6 +351,10 @@ func (rm *RoleManagerImpl) HasLink(name1 string, name2 string, domains ...string
 		return true, nil
 	}
 
+	// Lock to prevent race conditions between getRole and removeRole
+	rm.mutex.Lock()
+	defer rm.mutex.Unlock()
+
 	user, userCreated := rm.getRole(name1)
 	role, roleCreated := rm.getRole(name2)
 
@@ -400,6 +406,80 @@ func (rm *RoleManagerImpl) GetUsers(name string, domain ...string) ([]string, er
 	return role.getUsers(), nil
 }
 
+// GetImplicitRoles gets the implicit roles that a user inherits, respecting maxHierarchyLevel.
+func (rm *RoleManagerImpl) GetImplicitRoles(name string, domain ...string) ([]string, error) {
+	user, created := rm.getRole(name)
+	if created {
+		defer rm.removeRole(user.name)
+	}
+
+	var res []string
+	roleSet := make(map[string]bool)
+	roleSet[name] = true
+	roles := map[string]*Role{user.name: user}
+
+	return rm.getImplicitRolesHelper(roles, roleSet, res, 0), nil
+}
+
+// GetImplicitUsers gets the implicit users that inherits a role, respecting maxHierarchyLevel.
+func (rm *RoleManagerImpl) GetImplicitUsers(name string, domain ...string) ([]string, error) {
+	role, created := rm.getRole(name)
+	if created {
+		defer rm.removeRole(role.name)
+	}
+
+	var res []string
+	userSet := make(map[string]bool)
+	userSet[name] = true
+	users := map[string]*Role{role.name: role}
+
+	return rm.getImplicitUsersHelper(users, userSet, res, 0), nil
+}
+
+// getImplicitRolesHelper is a helper function for GetImplicitRoles that respects maxHierarchyLevel.
+func (rm *RoleManagerImpl) getImplicitRolesHelper(roles map[string]*Role, roleSet map[string]bool, res []string, level int) []string {
+	if level >= rm.maxHierarchyLevel || len(roles) == 0 {
+		return res
+	}
+
+	nextRoles := map[string]*Role{}
+	for _, role := range roles {
+		role.rangeRoles(func(key, value interface{}) bool {
+			roleName := key.(string)
+			if _, ok := roleSet[roleName]; !ok {
+				res = append(res, roleName)
+				roleSet[roleName] = true
+				nextRoles[roleName] = value.(*Role)
+			}
+			return true
+		})
+	}
+
+	return rm.getImplicitRolesHelper(nextRoles, roleSet, res, level+1)
+}
+
+// getImplicitUsersHelper is a helper function for GetImplicitUsers that respects maxHierarchyLevel.
+func (rm *RoleManagerImpl) getImplicitUsersHelper(users map[string]*Role, userSet map[string]bool, res []string, level int) []string {
+	if level >= rm.maxHierarchyLevel || len(users) == 0 {
+		return res
+	}
+
+	nextUsers := map[string]*Role{}
+	for _, user := range users {
+		user.rangeUsers(func(key, value interface{}) bool {
+			userName := key.(string)
+			if _, ok := userSet[userName]; !ok {
+				res = append(res, userName)
+				userSet[userName] = true
+				nextUsers[userName] = value.(*Role)
+			}
+			return true
+		})
+	}
+
+	return rm.getImplicitUsersHelper(nextUsers, userSet, res, level+1)
+}
+
 func (rm *RoleManagerImpl) toString() []string {
 	var roles []string
 
@@ -654,6 +734,26 @@ func (dm *DomainManager) GetUsers(name string, domains ...string) ([]string, err
 	return rm.GetUsers(name, domains...)
 }
 
+// GetImplicitRoles gets the implicit roles that a subject inherits, respecting maxHierarchyLevel.
+func (dm *DomainManager) GetImplicitRoles(name string, domains ...string) ([]string, error) {
+	domain, err := dm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	rm := dm.getRoleManager(domain, false)
+	return rm.GetImplicitRoles(name, domains...)
+}
+
+// GetImplicitUsers gets the implicit users that inherits a role, respecting maxHierarchyLevel.
+func (dm *DomainManager) GetImplicitUsers(name string, domains ...string) ([]string, error) {
+	domain, err := dm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	rm := dm.getRoleManager(domain, false)
+	return rm.GetImplicitUsers(name, domains...)
+}
+
 func (dm *DomainManager) toString() []string {
 	var roles []string
 
@@ -712,6 +812,12 @@ func (dm *DomainManager) BuildRelationship(name1 string, name2 string, domain ..
 	return nil
 }
 
+// DeleteDomain deletes the specified domain from DomainManager.
+func (dm *DomainManager) DeleteDomain(domain string) error {
+	dm.rmMap.Delete(domain)
+	return nil
+}
+
 type RoleManager struct {
 	*DomainManager
 }
@@ -722,6 +828,11 @@ func NewRoleManager(maxHierarchyLevel int) *RoleManager {
 	return rm
 }
 
+// DeleteDomain does nothing for RoleManagerImpl (no domain concept).
+func (rm *RoleManagerImpl) DeleteDomain(domain string) error {
+	return errors.New("DeleteDomain is not supported by RoleManagerImpl (no domain concept)")
+}
+
 type ConditionalRoleManager struct {
 	RoleManagerImpl
 }
@@ -756,6 +867,10 @@ func (crm *ConditionalRoleManager) HasLink(name1 string, name2 string, domains .
 		return true, nil
 	}
 
+	// Lock to prevent race conditions between getRole and removeRole
+	crm.mutex.Lock()
+	defer crm.mutex.Unlock()
+
 	user, userCreated := crm.getRole(name1)
 	role, roleCreated := crm.getRole(name2)
 
@@ -790,20 +905,7 @@ func (crm *ConditionalRoleManager) hasLinkHelper(targetName string, roles map[st
 }
 
 func (crm *ConditionalRoleManager) getNextRoles(currentRole, nextRole *Role, domains []string, nextRoles map[string]*Role) bool {
-	passLinkConditionFunc := true
-	var err error
-	// If LinkConditionFunc exists, it needs to pass the verification to get nextRole
-	if len(domains) == 0 {
-		if linkConditionFunc, existLinkCondition := crm.GetLinkConditionFunc(currentRole.name, nextRole.name); existLinkCondition {
-			params, _ := crm.GetLinkConditionFuncParams(currentRole.name, nextRole.name)
-			passLinkConditionFunc, err = linkConditionFunc(params...)
-		}
-	} else {
-		if linkConditionFunc, existLinkCondition := crm.GetDomainLinkConditionFunc(currentRole.name, nextRole.name, domains[0]); existLinkCondition {
-			params, _ := crm.GetLinkConditionFuncParams(currentRole.name, nextRole.name, domains[0])
-			passLinkConditionFunc, err = linkConditionFunc(params...)
-		}
-	}
+	passLinkConditionFunc, err := crm.checkLinkCondition(currentRole.name, nextRole.name, domains)
 
 	if err != nil {
 		crm.logger.LogError(err, "hasLinkHelper LinkCondition Error")
@@ -817,6 +919,163 @@ func (crm *ConditionalRoleManager) getNextRoles(currentRole, nextRole *Role, dom
 	return true
 }
 
+func (crm *ConditionalRoleManager) checkLinkCondition(name1, name2 string, domain []string) (bool, error) {
+	passLinkConditionFunc := true
+	var err error
+
+	if len(domain) == 0 {
+		if linkConditionFunc, existLinkCondition := crm.GetLinkConditionFunc(name1, name2); existLinkCondition {
+			params, _ := crm.GetLinkConditionFuncParams(name1, name2)
+			passLinkConditionFunc, err = linkConditionFunc(params...)
+		}
+	} else {
+		if linkConditionFunc, existLinkCondition := crm.GetDomainLinkConditionFunc(name1, name2, domain[0]); existLinkCondition {
+			params, _ := crm.GetLinkConditionFuncParams(name1, name2, domain[0])
+			passLinkConditionFunc, err = linkConditionFunc(params...)
+		}
+	}
+
+	return passLinkConditionFunc, err
+}
+
+func (crm *ConditionalRoleManager) GetRoles(name string, domains ...string) ([]string, error) {
+	user, created := crm.getRole(name)
+	if created {
+		defer crm.removeRole(user.name)
+	}
+	var roles []string
+	user.rangeRoles(func(key, value interface{}) bool {
+		roleName := key.(string)
+		passLinkConditionFunc, err := crm.checkLinkCondition(name, roleName, domains)
+		if err != nil {
+			crm.logger.LogError(err, "getRoles LinkCondition Error")
+			return true
+		}
+
+		if passLinkConditionFunc {
+			roles = append(roles, roleName)
+		}
+
+		return true
+	})
+	return roles, nil
+}
+
+func (crm *ConditionalRoleManager) GetUsers(name string, domains ...string) ([]string, error) {
+	role, created := crm.getRole(name)
+	if created {
+		defer crm.removeRole(name)
+	}
+	var users []string
+	role.rangeUsers(func(key, value interface{}) bool {
+		userName := key.(string)
+
+		passLinkConditionFunc, err := crm.checkLinkCondition(userName, name, domains)
+		if err != nil {
+			crm.logger.LogError(err, "getUsers LinkCondition Error")
+			return true
+		}
+
+		if passLinkConditionFunc {
+			users = append(users, userName)
+		}
+
+		return true
+	})
+
+	return users, nil
+}
+
+// GetImplicitRoles gets the implicit roles that a user inherits, respecting maxHierarchyLevel and link conditions.
+func (crm *ConditionalRoleManager) GetImplicitRoles(name string, domain ...string) ([]string, error) {
+	user, created := crm.getRole(name)
+	if created {
+		defer crm.removeRole(user.name)
+	}
+
+	var res []string
+	roleSet := make(map[string]bool)
+	roleSet[name] = true
+	roles := map[string]*Role{user.name: user}
+
+	return crm.getImplicitRolesHelper(roles, roleSet, res, 0, domain), nil
+}
+
+// GetImplicitUsers gets the implicit users that inherits a role, respecting maxHierarchyLevel and link conditions.
+func (crm *ConditionalRoleManager) GetImplicitUsers(name string, domain ...string) ([]string, error) {
+	role, created := crm.getRole(name)
+	if created {
+		defer crm.removeRole(role.name)
+	}
+
+	var res []string
+	userSet := make(map[string]bool)
+	userSet[name] = true
+	users := map[string]*Role{role.name: role}
+
+	return crm.getImplicitUsersHelper(users, userSet, res, 0, domain), nil
+}
+
+// getImplicitRolesHelper is a helper function for GetImplicitRoles that respects maxHierarchyLevel and link conditions.
+func (crm *ConditionalRoleManager) getImplicitRolesHelper(roles map[string]*Role, roleSet map[string]bool, res []string, level int, domains []string) []string {
+	if level >= crm.maxHierarchyLevel || len(roles) == 0 {
+		return res
+	}
+
+	nextRoles := map[string]*Role{}
+	for _, role := range roles {
+		role.rangeRoles(func(key, value interface{}) bool {
+			roleName := key.(string)
+			if _, ok := roleSet[roleName]; !ok {
+				passLinkConditionFunc, err := crm.checkLinkCondition(role.name, roleName, domains)
+				if err != nil {
+					crm.logger.LogError(err, "getImplicitRoles LinkCondition Error")
+					return true
+				}
+
+				if passLinkConditionFunc {
+					res = append(res, roleName)
+					roleSet[roleName] = true
+					nextRoles[roleName] = value.(*Role)
+				}
+			}
+			return true
+		})
+	}
+
+	return crm.getImplicitRolesHelper(nextRoles, roleSet, res, level+1, domains)
+}
+
+// getImplicitUsersHelper is a helper function for GetImplicitUsers that respects maxHierarchyLevel and link conditions.
+func (crm *ConditionalRoleManager) getImplicitUsersHelper(users map[string]*Role, userSet map[string]bool, res []string, level int, domains []string) []string {
+	if level >= crm.maxHierarchyLevel || len(users) == 0 {
+		return res
+	}
+
+	nextUsers := map[string]*Role{}
+	for _, user := range users {
+		user.rangeUsers(func(key, value interface{}) bool {
+			userName := key.(string)
+			if _, ok := userSet[userName]; !ok {
+				passLinkConditionFunc, err := crm.checkLinkCondition(userName, user.name, domains)
+				if err != nil {
+					crm.logger.LogError(err, "getImplicitUsers LinkCondition Error")
+					return true
+				}
+
+				if passLinkConditionFunc {
+					res = append(res, userName)
+					userSet[userName] = true
+					nextUsers[userName] = value.(*Role)
+				}
+			}
+			return true
+		})
+	}
+
+	return crm.getImplicitUsersHelper(nextUsers, userSet, res, level+1, domains)
+}
+
 // GetLinkConditionFunc get LinkConditionFunc based on userName, roleName.
 func (crm *ConditionalRoleManager) GetLinkConditionFunc(userName, roleName string) (rbac.LinkConditionFunc, bool) {
 	return crm.GetDomainLinkConditionFunc(userName, roleName, defaultDomain)
@@ -949,6 +1208,42 @@ func (cdm *ConditionalDomainManager) HasLink(name1 string, name2 string, domains
 	return rm.HasLink(name1, name2, domains...)
 }
 
+func (cdm *ConditionalDomainManager) GetRoles(name string, domains ...string) ([]string, error) {
+	domain, err := cdm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	crm := cdm.getConditionalRoleManager(domain, false)
+	return crm.GetRoles(name, domains...)
+}
+
+func (cdm *ConditionalDomainManager) GetUsers(name string, domains ...string) ([]string, error) {
+	domain, err := cdm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	crm := cdm.getConditionalRoleManager(domain, false)
+	return crm.GetUsers(name, domains...)
+}
+
+func (cdm *ConditionalDomainManager) GetImplicitRoles(name string, domains ...string) ([]string, error) {
+	domain, err := cdm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	crm := cdm.getConditionalRoleManager(domain, false)
+	return crm.GetImplicitRoles(name, domains...)
+}
+
+func (cdm *ConditionalDomainManager) GetImplicitUsers(name string, domains ...string) ([]string, error) {
+	domain, err := cdm.getDomain(domains...)
+	if err != nil {
+		return nil, err
+	}
+	crm := cdm.getConditionalRoleManager(domain, false)
+	return crm.GetImplicitUsers(name, domains...)
+}
+
 // AddLink adds the inheritance link between role: name1 and role: name2.
 // aka role: name1 inherits role: name2.
 func (cdm *ConditionalDomainManager) AddLink(name1 string, name2 string, domains ...string) error {
@@ -1012,3 +1307,29 @@ func (cdm *ConditionalDomainManager) SetDomainLinkConditionFuncParams(userName,
 		return true
 	})
 }
+
+// AddDomainMatchingFunc support use domain pattern in g.
+func (cdm *ConditionalDomainManager) AddDomainMatchingFunc(name string, fn rbac.MatchingFunc) {
+	cdm.domainMatchingFunc = fn
+	cdm.rmMap.Range(func(key, value interface{}) bool {
+		value.(*ConditionalRoleManager).AddDomainMatchingFunc(name, fn)
+		return true
+	})
+	cdm.rebuild()
+}
+
+// rebuild clears the map of ConditionalRoleManagers.
+func (cdm *ConditionalDomainManager) rebuild() {
+	rmMap := cdm.rmMap
+	_ = cdm.Clear()
+	rmMap.Range(func(key, value interface{}) bool {
+		domain := key.(string)
+		crm := value.(*ConditionalRoleManager)
+
+		crm.Range(func(name1, name2 string, _ ...string) bool {
+			_ = cdm.AddLink(name1, name2, domain)
+			return true
+		})
+		return true
+	})
+}
