@@ -15,12 +15,14 @@
 package casbin
 
 import (
+	"fmt"
 	"log"
 	"sort"
 	"testing"
 
 	"github.com/casbin/casbin/v2/constant"
 	"github.com/casbin/casbin/v2/errors"
+	defaultrolemanager "github.com/casbin/casbin/v2/rbac/default-role-manager"
 	"github.com/casbin/casbin/v2/util"
 )
 
@@ -633,6 +635,43 @@ func TestGetImplicitUsersForResource(t *testing.T) {
 		{"alice", "data2", "write"}}, "data2")
 }
 
+func TestGetImplicitUsersForResourceWithResourceRoles(t *testing.T) {
+	e, _ := NewEnforcer("examples/rbac_with_resource_roles_model.conf", "examples/rbac_with_resource_roles_policy.csv")
+
+	// Test data1 resource - should return users who have access through g2 relationships
+	data1Users, err := e.GetNamedImplicitUsersForResource("g2", "data1")
+	if err != nil {
+		t.Fatalf("GetNamedImplicitUsersForResource failed: %v", err)
+	}
+
+	expectedData1Users := 2 // [alice data1 read] + [alice data_group write]
+	if len(data1Users) != expectedData1Users {
+		t.Errorf("Expected %d users for data1 resource, got %d: %v", expectedData1Users, len(data1Users), data1Users)
+	}
+
+	// Test data2 resource - should return users who have access through g2 relationships
+	data2Users, err := e.GetNamedImplicitUsersForResource("g2", "data2")
+	if err != nil {
+		t.Fatalf("GetNamedImplicitUsersForResource failed: %v", err)
+	}
+
+	expectedData2Users := 2 // [bob data2 write] + [alice data_group write]
+	if len(data2Users) != expectedData2Users {
+		t.Errorf("Expected %d users for data2 resource, got %d: %v", expectedData2Users, len(data2Users), data2Users)
+	}
+
+	// Test with "g" policy type - should return users who have access through g relationships
+	data1UsersG, err := e.GetNamedImplicitUsersForResource("g", "data1")
+	if err != nil {
+		t.Fatalf("GetNamedImplicitUsersForResource with g failed: %v", err)
+	}
+
+	expectedData1UsersG := 1 // [alice data1 read] only
+	if len(data1UsersG) != expectedData1UsersG {
+		t.Errorf("Expected %d users for data1 resource with g policy, got %d: %v", expectedData1UsersG, len(data1UsersG), data1UsersG)
+	}
+}
+
 func testGetImplicitUsersForResourceByDomain(t *testing.T, e *Enforcer, res [][]string, resource string, domain string) {
 	t.Helper()
 	myRes, err := e.GetImplicitUsersForResourceByDomain(resource, domain)
@@ -657,3 +696,188 @@ func TestGetImplicitUsersForResourceByDomain(t *testing.T) {
 	testGetImplicitUsersForResourceByDomain(t, e, [][]string{{"bob", "domain2", "data2", "read"},
 		{"bob", "domain2", "data2", "write"}}, "data2", "domain2")
 }
+
+func TestConditional(t *testing.T) {
+	e, _ := NewEnforcer("examples/rbac_with_domains_conditional_model.conf", "examples/rbac_with_domains_conditional_policy.csv")
+	g, _ := e.GetNamedGroupingPolicy("g")
+	for _, gp := range g {
+		e.AddNamedDomainLinkConditionFunc("g", gp[0], gp[1], gp[2], util.TimeMatchFunc)
+	}
+
+	testDomainEnforce(t, e, "alice", "domain1", "service1", "/list", true)
+	testDomainEnforce(t, e, "bob", "domain2", "service2", "/broadcast", true)
+	testDomainEnforce(t, e, "jack", "domain1", "service1", "/list", false)
+	testGetImplicitRolesInDomain(t, e, "alice", "domain1", []string{"test1"})
+	testGetRolesInDomain(t, e, "alice", "domain1", []string{"test1"})
+	testGetUsersInDomain(t, e, "test1", "domain1", []string{"alice"})
+}
+
+func TestMaxHierarchyLevelConsistency(t *testing.T) {
+	// Test consistency behavior under different maxHierarchyLevel values
+	testCases := []struct {
+		maxLevel int
+		name     string
+	}{
+		{1, "maxHierarchyLevel=1"},
+		{2, "maxHierarchyLevel=2"},
+		{3, "maxHierarchyLevel=3"},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			// Use model files from examples
+			e, err := NewEnforcer("examples/rbac_model.conf", "examples/rbac_policy.csv")
+			if err != nil {
+				t.Fatalf("Failed to create enforcer: %v", err)
+			}
+
+			// Set the maximum hierarchy level for role manager
+			rm := defaultrolemanager.NewRoleManager(tc.maxLevel)
+			e.SetRoleManager(rm)
+
+			// Add role hierarchy: level0 -> level1 -> level2 -> level3 -> level4
+			_, err = e.AddRoleForUser("level0", "level1")
+			if err != nil {
+				t.Fatalf("Failed to add role for user: %v", err)
+			}
+			_, err = e.AddRoleForUser("level1", "level2")
+			if err != nil {
+				t.Fatalf("Failed to add role for user: %v", err)
+			}
+			_, err = e.AddRoleForUser("level2", "level3")
+			if err != nil {
+				t.Fatalf("Failed to add role for user: %v", err)
+			}
+			_, err = e.AddRoleForUser("level3", "level4")
+			if err != nil {
+				t.Fatalf("Failed to add role for user: %v", err)
+			}
+
+			// Test HasLink method
+			t.Run("HasLink", func(t *testing.T) {
+				for i := 1; i <= 4; i++ {
+					hasLink, err := rm.HasLink("level0", fmt.Sprintf("level%d", i))
+					if err != nil {
+						t.Fatalf("HasLink error: %v", err)
+					}
+					expected := i <= tc.maxLevel
+					if hasLink != expected {
+						t.Errorf("HasLink(level0, level%d): got %v, want %v", i, hasLink, expected)
+					}
+				}
+			})
+
+			// Test GetImplicitRolesForUser method
+			t.Run("GetImplicitRolesForUser", func(t *testing.T) {
+				implicitRoles, err := e.GetImplicitRolesForUser("level0")
+				if err != nil {
+					t.Fatalf("GetImplicitRolesForUser error: %v", err)
+				}
+
+				expectedCount := tc.maxLevel
+				if len(implicitRoles) != expectedCount {
+					t.Errorf("GetImplicitRolesForUser(level0): got %d roles %v, want %d roles",
+						len(implicitRoles), implicitRoles, expectedCount)
+				}
+
+				// Verify that returned roles are correct
+				for i := 1; i <= tc.maxLevel; i++ {
+					expectedRole := fmt.Sprintf("level%d", i)
+					found := false
+					for _, role := range implicitRoles {
+						if role == expectedRole {
+							found = true
+							break
+						}
+					}
+					if !found {
+						t.Errorf("Expected role %s not found in implicit roles: %v", expectedRole, implicitRoles)
+					}
+				}
+			})
+
+			// Test GetImplicitUsersForRole method
+			t.Run("GetImplicitUsersForRole", func(t *testing.T) {
+				implicitUsers, err := e.GetImplicitUsersForRole("level4")
+				if err != nil {
+					t.Fatalf("GetImplicitUsersForRole error: %v", err)
+				}
+
+				expectedCount := tc.maxLevel
+				if len(implicitUsers) != expectedCount {
+					t.Errorf("GetImplicitUsersForRole(level4): got %d users %v, want %d users",
+						len(implicitUsers), implicitUsers, expectedCount)
+				}
+
+				// Verify that returned users are correct (starting from level3 upward)
+				for i := 0; i < tc.maxLevel; i++ {
+					expectedUser := fmt.Sprintf("level%d", 3-i)
+					found := false
+					for _, user := range implicitUsers {
+						if user == expectedUser {
+							found = true
+							break
+						}
+					}
+					if !found {
+						t.Errorf("Expected user %s not found in implicit users: %v", expectedUser, implicitUsers)
+					}
+				}
+			})
+
+			// Test implicit roles for different users
+			t.Run("DifferentUsersImplicitRoles", func(t *testing.T) {
+				for i := 0; i <= 3; i++ {
+					user := fmt.Sprintf("level%d", i)
+					implicitRoles, err := e.GetImplicitRolesForUser(user)
+					if err != nil {
+						t.Fatalf("GetImplicitRolesForUser(%s) error: %v", user, err)
+					}
+
+					// Verify that the number of returned roles does not exceed maxHierarchyLevel
+					if len(implicitRoles) > tc.maxLevel {
+						t.Errorf("GetImplicitRolesForUser(%s): got %d roles, should not exceed maxHierarchyLevel %d",
+							user, len(implicitRoles), tc.maxLevel)
+					}
+				}
+			})
+		})
+	}
+}
+
+func testGetImplicitObjectPatternsForUser(t *testing.T, e *Enforcer, user string, domain string, action string, res []string) {
+	t.Helper()
+	myRes, err := e.GetImplicitObjectPatternsForUser(user, domain, action)
+	if err != nil {
+		t.Error("Implicit object patterns for ", user, " under domain ", domain, " with action ", action, " could not be fetched: ", err.Error())
+	}
+	t.Log("Implicit object patterns for ", user, " under domain ", domain, " with action ", action, ": ", myRes)
+
+	if !util.SetEquals(res, myRes) {
+		t.Error("Implicit object patterns for ", user, " under domain ", domain, " with action ", action, ": ", myRes, ", supposed to be ", res)
+	}
+}
+
+func TestGetImplicitObjectPatternsForUser(t *testing.T) {
+	// Test with domain pattern model
+	e, _ := NewEnforcer("examples/rbac_with_domain_pattern_model.conf", "examples/rbac_with_domain_pattern_policy.csv")
+	e.AddNamedDomainMatchingFunc("g", "KeyMatch", util.KeyMatch)
+
+	// Test case 1: admin user with wildcard domain access
+	testGetImplicitObjectPatternsForUser(t, e, "admin", "domain1", "read", []string{"data1", "data3"})
+	testGetImplicitObjectPatternsForUser(t, e, "admin", "domain1", "write", []string{"data1"})
+
+	// Test case 2: alice user inheriting admin role in domain2
+	testGetImplicitObjectPatternsForUser(t, e, "alice", "domain2", "read", []string{"data2", "data3"})
+	testGetImplicitObjectPatternsForUser(t, e, "alice", "domain2", "write", []string{"data2"})
+
+	// Test case 3: bob user with specific domain access
+	testGetImplicitObjectPatternsForUser(t, e, "bob", "domain2", "read", []string{"data2", "data3"})
+	testGetImplicitObjectPatternsForUser(t, e, "bob", "domain2", "write", []string{"data2"})
+
+	// Test case 4: non-existent domain (admin has wildcard access to data3)
+	testGetImplicitObjectPatternsForUser(t, e, "admin", "non_existent", "read", []string{"data3"})
+
+	// Test case 5: non-existent action
+	testGetImplicitObjectPatternsForUser(t, e, "admin", "domain1", "non_existent", []string{})
+}
